//////////////////////////////////////////////////////////////////////////////
// nitrogenmenuengine.cpp
// stores event filters and maps widgets to timelines for animations
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenmenuengine.h"
#include "nitrogenmenuengine.moc"

#include <QtCore/QEvent>

namespace Nitrogen
{

  //____________________________________________________________
  void MenuEngineV1::registerWidget( QWidget* widget )
  {

    if( !( enabled() && widget ) ) return;
    
    // create new data class
    if( !data_.contains( widget ) ) data_.insert( widget, QPointer<MenuDataV1>( new MenuDataV1( this, widget, maxFrame(), duration() ) ) );

    // connect destruction signal
    disconnect( widget, SIGNAL( destroyed( QObject* ) ), this, SLOT( unregisterWidget( QObject* ) ) );
    connect( widget, SIGNAL( destroyed( QObject* ) ), this, SLOT( unregisterWidget( QObject* ) ) );

  }

  //____________________________________________________________
  TimeLine::Pointer MenuEngineV1::timeLine( const QObject* object )
  {

    if( !enabled() ) return TimeLine::Pointer();
    QPointer<MenuDataV1> out( data_.find( object ) );
    return out ? out->currentTimeLine():TimeLine::Pointer();

  }

  //____________________________________________________________
  TimeLine::Pointer MenuEngineV1::previousTimeLine( const QObject* object )
  {

    if( !enabled() ) return TimeLine::Pointer();
    QPointer<MenuDataV1> out( data_.find( object ) );
    return out ? out->previousTimeLine():TimeLine::Pointer();

  }

  //____________________________________________________________
  QRect MenuEngineV1::currentRect( const QObject* object )
  {

    if( !enabled() ) return QRect();
    QPointer<MenuDataV1> out( data_.find( object ) );
    return out ? out->currentRect():QRect();

  }

  //____________________________________________________________
  QRect MenuEngineV1::previousRect( const QObject* object )
  {
    if( !enabled() ) return QRect();
    QPointer<MenuDataV1> out( data_.find( object ) );
    return out ? out->previousRect():QRect();

  }

  //____________________________________________________________
  void MenuEngineV2::registerWidget( QWidget* widget )
  {

    // create new data class
    if( !data_.contains( widget ) ) data_.insert( widget, QPointer<MenuDataV2>( new MenuDataV2( this, widget, maxFrame(), duration() ) ) );

    // connect destruction signal
    disconnect( widget, SIGNAL( destroyed( QObject* ) ), this, SLOT( unregisterWidget( QObject* ) ) );
    connect( widget, SIGNAL( destroyed( QObject* ) ), this, SLOT( unregisterWidget( QObject* ) ) );

  }

  //____________________________________________________________
  TimeLine::Pointer MenuEngineV2::timeLine( const QObject* object )
  {

    if( !enabled() ) return TimeLine::Pointer();
    QPointer<MenuDataV2> out( data_.find( object ) );
    return out ? out->timeLine():TimeLine::Pointer();

  }

  //____________________________________________________________
  TimeLine::Pointer MenuEngineV2::previousTimeLine( const QObject* object )
  {

    if( !enabled() ) return TimeLine::Pointer();
    QPointer<MenuDataV2> out( data_.find( object ) );
    /*
    for 'follow mouse' animation, current and previous timeline are identical.
    timeline is interpreted as "previous" when it is running backward
    */
    return
      ( out && out->timeLine()->direction() == QTimeLine::Backward ) ?
      out->timeLine():TimeLine::Pointer();

  }

  //____________________________________________________________
  QRect MenuEngineV2::currentRect( const QObject* object )
  {
    if( !enabled() ) return QRect();
    QPointer<MenuDataV2> out( data_.find( object ) );
    return out ? out->currentRect():QRect();

  }

  //____________________________________________________________
  QRect MenuEngineV2::previousRect( const QObject* object )
  {
    if( !enabled() ) return QRect();
    QPointer<MenuDataV2> out( data_.find( object ) );

    // for 'follow mouse' animation, current and previous rect are identical.
    return out ? out->currentRect():QRect();

  }

  //____________________________________________________________
  QRect MenuEngineV2::animatedRect( const QObject* object )
  {
    if( !enabled() ) return QRect();
    QPointer<MenuDataV2> out( data_.find( object ) );
    return out ? out->animatedRect():QRect();

  }

  //____________________________________________________________
  bool MenuEngineV2::isTimerActive( const QObject* object )
  {
    if( !enabled() ) return false;
    QPointer<MenuDataV2> out( data_.find( object ) );
    return out ? out->timer().isActive():false;

  }
}
