//////////////////////////////////////////////////////////////////////////////
// nitrogenstackedwidgettransitionengine.cpp
// handles smooth transitions for QStackedWidget
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogenstackedwidgettransitionengine.h"
#include "nitrogenstackedwidgettransitionengine.moc"

#include <QApplication>

namespace Nitrogen
{

  //________________________________________________________
  StackedWidgetTransitionEngine::StackedWidgetTransitionEngine( QStackedWidget* parent, int maxFrame, int duration ):
    BaseTransitionEngine( parent, maxFrame, duration ),
    target_( parent )
  {
    createTransitionWidget();

    // install event filter
    target_->installEventFilter( this );

    // loop over existing widgets
    for( int index = 0; index < target_->count(); index++ )
    {
      if( QWidget* widget = target_->widget( index ) )
      { childAddedEvent( widget ); }
    }

  }

  //________________________________________________________
  bool StackedWidgetTransitionEngine::eventFilter( QObject* object, QEvent* event )
  {

    if( !object ) return false;

    if( object == target_ )
    {
      switch( event->type() )
      {

        case QEvent::ChildAdded:
        {
          QChildEvent* childEvent( static_cast<QChildEvent*>( event ) );
          childAddedEvent( childEvent->child() );
          break;
        }

        default: break;

      }

    } else if( object->parent() == target_ ) {

      switch( event->type() )
      {

        case QEvent::Show:
        childShownEvent( object );
        break;

        case QEvent::Hide:
        childHiddenEvent( object );
        return false;
        break;

        default: break;

      }

    }

    return false;

  }

  //__________________________________________________________
  void StackedWidgetTransitionEngine::createTransitionWidget( void )
  {

    transitionWidget_ = TransitionWidget::Pointer( new TransitionWidget( target_, maxFrame(), duration() ) );
    transitionWidget_->setEnabled( enabled() );
    transitionWidget_->hide();

    connect( transitionWidget_->timeLine().data(), SIGNAL( finished() ), SLOT( transitionFinished() ) );
    connect( transitionWidget_.data(), SIGNAL( destroyed() ), SLOT( createTransitionWidget() ) );

  }

  //___________________________________________________________________
  void StackedWidgetTransitionEngine::transitionFinished( void )
  {
    if( !transitionWidget_ ) return;

    transitionWidget_->setParent( target_ );
    transitionWidget_->hide();
    target_->update();

  }

  //__________________________________________________________
  void StackedWidgetTransitionEngine::childAddedEvent( QObject* object )
  {

    // do not add transition widget
    if( object == transitionWidget_ ) return;
    if( object->parent() != target_ ) return;
    QWidget* widget( qobject_cast<QWidget*>( object ) );
    if( !widget ) return;

    // add event filter
    widget->removeEventFilter( this );
    widget->installEventFilter( this );

    if( !widgetGrabs_.contains( object ) )
    {
      WidgetGrab::Pointer widgetGrab( new WidgetGrab( widget ) );
      widgetGrabs_.insert( object, widgetGrab );
      connect( object, SIGNAL( destroyed( QObject* ) ), SLOT( unregisterWidget( QObject* ) ) );
    }

  }

  //__________________________________________________________
  void StackedWidgetTransitionEngine::childShownEvent( QObject* object )
  {
    if( object == transitionWidget_ ) return;
    if( object->parent() != target_ ) return;

    QWidget* widget( qobject_cast<QWidget*>( object ) );
    if( !widget ) return;

    if( transitionWidget_ )
    {

      if( lastHiddenObject_ != object )
      {
        transitionWidget_->setParent( widget );
        transitionWidget_->show();
        transitionWidget_->start();
      }
    }
    return;

  }


  //__________________________________________________________
  void StackedWidgetTransitionEngine::childHiddenEvent( QObject* object )
  {

    if( object == transitionWidget_ ) return;
    if( object->parent() != target_ ) return;

    QWidget* widget( qobject_cast<QWidget*>( object ) );
    if( !widget ) return;

    if( lastHiddenObject_ == object ) return;
    if( transitionWidget_ )
    {
      WidgetGrab::Pointer widgetGrab( widgetGrabs_.value( object ) );
      if( !widgetGrab ) return;
      transitionWidget_->initialize( widgetGrab->pixmap(), widget->rect() );
      lastHiddenObject_ = object;
    }

  }

}
