//////////////////////////////////////////////////////////////////////////////
// nitrogentoolbardata.cpp
// stores event filters and maps widgets to timelines for animations
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogentoolbardata.h"
#include "nitrogentoolbardata.moc"

#include<QtGui/QToolButton>
namespace Nitrogen
{


  //________________________________________________________________________
  ToolBarData::ToolBarData( QObject* parent, QWidget* target, int maxFrame, int duration ):
    WidgetData( parent, target ),
    timeLine_( new TimeLine( duration, this ) )
  {

    // setup timeLine
    timeLine()->setFrameRange( 0, maxFrame );
    timeLine()->setCurveShape( QTimeLine::LinearCurve );
    connect( timeLine().data(), SIGNAL( frameChanged( int ) ), SLOT( updateAnimatedRect( int ) ) );
    connect( timeLine().data(), SIGNAL( frameChanged( int ) ), target, SLOT( update() ) );
    connect( timeLine().data(), SIGNAL( finished() ), target, SLOT( update() ) );

    // add all children widgets to event handler
    foreach( QWidget* child, target->findChildren<QWidget*>() )
    { if( child->parentWidget() == target ) childAddedEvent( child ); }

  }

  //______________________________________________
  bool ToolBarData::eventFilter( QObject* object, QEvent* event )
  {
    if( !enabled() ) return false;

    // check object
    if( object == target() )
    {

      switch( event->type() )
      {

        case QEvent::Enter:
        object->event( event );
        enterEvent( object );
        return true;

        case QEvent::ChildAdded:
        {
          QChildEvent* childEvent( static_cast<QChildEvent*>( event ) );
          childAddedEvent( childEvent->child() );
          break;
        }

        default: break;

      }

    } else if( object->parent() == target() ) {

      switch( event->type() )
      {

        case QEvent::Enter:
        childEnterEvent( object );
        break;

        case QEvent::Leave:
        if( currentObject() && !timer_.isActive() ) timer_.start( 100, this );
        break;

        default:
        break;

      }

    }

    return false;

  }

  //____________________________________________________________
  void ToolBarData::updateAnimatedRect( int frame )
  {

    // check rect validity
    if( currentRect().isNull() || previousRect().isNull() )
    {
      animatedRect_ = QRect();
      return;
    }

    // compute rect located 'between' previous and current
    const qreal ratio( qreal(frame)/qreal(timeLine()->endFrame()) );
    animatedRect_.setLeft( previousRect().left() + ratio*(currentRect().left() - previousRect().left()) );
    animatedRect_.setRight( previousRect().right() + ratio*(currentRect().right() - previousRect().right()) );
    animatedRect_.setTop( previousRect().top() + ratio*(currentRect().top() - previousRect().top()) );
    animatedRect_.setBottom( previousRect().bottom() + ratio*(currentRect().bottom() - previousRect().bottom()) );
    return;

  }

  //________________________________________________________________________
  void ToolBarData::enterEvent( const QObject* )
  {

    if( timer_.isActive() ) timer_.stop();
    if( timeLine()->isRunning() ) timeLine()->stop();
    clearPreviousRect();
    clearAnimatedRect();
    return;

  }


  //________________________________________________________________________
  void ToolBarData::leaveEvent( const QObject* )
  {

    clearAnimatedRect();
    clearPreviousRect();
    if( currentObject() )
    {

      if( timeLine()->isRunning() ) timeLine()->stop();
      clearCurrentObject();
      timeLine()->setDirection( QTimeLine::Backward );
      timeLine()->start();

    }

    return;

  }

  //________________________________________________________________________
  void ToolBarData::childEnterEvent( const QObject* object )
  {

    if( object == currentObject() ) return;

    const QToolButton* local = qobject_cast<const QToolButton*>( object );

    // check if current position match another action
    if( local && local->isEnabled() )
    {

      if( timer_.isActive() ) timer_.stop();

      // update previous rect if the current action is valid
      if( currentObject() )
      {

        // if current action is valid,
        setPreviousRect( animatedRect() );
        if( previousRect().isNull() ) setPreviousRect( currentRect() );

      }

      // get rect
      QRect activeObjectRect( local->rect().translated( local->mapToParent( QPoint(0,0) ) ) );

      // update current action
      setCurrentObject( local );
      setCurrentRect( activeObjectRect );
      timeLine()->setDirection( QTimeLine::Forward );
      if( !timeLine()->isRunning() ) timeLine()->start();

    } else if( currentObject() ) {

      if( !timer_.isActive() ) timer_.start( 100, this );

    }

    return;

  }

  //___________________________________________________________________
  void ToolBarData::childAddedEvent( QObject* object )
  {
    QWidget* widget( qobject_cast<QWidget*>( object ) );
    if( !widget ) return;

    // add connections
    disconnect( timeLine().data(), SIGNAL( frameChanged( int ) ), widget, SLOT( update() ) );
    disconnect( timeLine().data(), SIGNAL( finished() ), widget, SLOT( update() ) );

    connect( timeLine().data(), SIGNAL( frameChanged( int ) ), widget, SLOT( update() ) );
    connect( timeLine().data(), SIGNAL( finished() ), widget, SLOT( update() ) );

    // add event filter
    widget->removeEventFilter( this );
    widget->installEventFilter( this );
  }

  //___________________________________________________________
  void ToolBarData::timerEvent( QTimerEvent *event )
  {

    if( event->timerId() != timer_.timerId() ) return WidgetData::timerEvent( event );
    timer_.stop();
    leaveEvent( target() );
  }

}
